<?php

namespace Drupal\status_dashboard_client\Controller;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ModuleExtensionList;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for Status dashboard client routes.
 */
class StatusDashboardClientController extends ControllerBase {

  /**
   * The module extension list.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected $moduleExtensionList;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs a new StatusDashboardClientController.
   *
   * @param \Drupal\Core\Extension\ModuleExtensionList $extension_list_module
   *   The module extension list.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(ModuleExtensionList $extension_list_module, TimeInterface $time) {
    $this->moduleExtensionList = $extension_list_module;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('extension.list.module'),
      $container->get('datetime.time')
    );
  }

  /**
   * Check the request.
   */
  public function doCheck(Request $request) {
    $projects_data = [];
    if ($available = update_get_available(TRUE)) {
      $this->moduleHandler()->loadInclude('update', 'compare.inc');
      $projects_data = update_calculate_project_data($available);
    }

    $security_updates = [];
    $feature_updates = [];

    $modules = $this->moduleExtensionList->getList();
    $modules_info = [];
    foreach ($projects_data as $project_data) {
      if (array_key_exists('security updates', $project_data) && !empty($project_data['security updates'])) {
        $security_updates[$project_data['title']] = $project_data['recommended'];
      }
      else {
        if ($project_data['latest_version'] !== $project_data['existing_version']) {
          $feature_updates[$project_data['title']] = $project_data['recommended'];
        }
      }
    }
    foreach ($modules as $module) {
      $modules_info[$module->info['name']] = $module->info['version'];
    }
    return new JsonResponse([
      'date' => $this->time->getCurrentTime(),
      'core' => \Drupal::VERSION,
      'modules' => $modules_info,
      'security_updates' => $security_updates,
      'feature_updates' => $feature_updates,
      'sitename' => $this->config('system.site')->get('name'),
    ], 200);

  }

}
